@@ -14,12 +14,17 @@ use strict;
 use warnings;
 
 use Class::XSAccessor
-    constructor => '_new',
     accessors   => {
-        name      => 'name',
+        # public
+        is_avail_on_bs => 'is_avail_on_bs',
+        is_local       => 'is_local',  # if module is available locally
+        name           => 'name',
+        prereqs        => 'prereqs',
+        # private
+        _blocking  => '_blocking',
+        _missing   => '_missing',
         _output    => '_output',
         _pkgname   => '_pkgname',
-        _prereqs   => '_prereqs',
         _rpm       => '_rpm',
         _srpm      => '_srpm',
         _wheel     => '_wheel',
@@ -50,51 +55,16 @@ my $rpm_locked = '';   # only one rpm transaction at a time
 #
 
 
-
-
-#--
-# CONSTRUCTOR
-
-sub spawn {
-    my ($class, $name) = @_;
-
-    # creating the object
-    my $obj = App::CPAN2Pkg::Module->_new(
-        name      => $name,
-        _prereqs  => [],
-        _wheel    => undef,
-    );
-
-    # spawning the session
-    my $session = POE::Session->create(
-        inline_states => {
-            # public events
-            available_on_bs    => \&available_on_bs,
-            build_upstream     => \&build_upstream,
-            cpan2dist          => \&cpan2dist,
-            find_prereqs       => \&find_prereqs,
-            import_upstream    => \&import_upstream,
-            install_from_dist  => \&install_from_dist,
-            install_from_local => \&install_from_local,
-            is_in_dist         => \&is_in_dist,
-            is_installed       => \&is_installed,
-            # private events
-            _build_upstream     => \&_build_upstream,
-            _cpan2dist          => \&_cpan2dist,
-            _find_prereqs       => \&_find_prereqs,
-            _import_upstream    => \&_import_upstream,
-            _install_from_dist  => \&_install_from_dist,
-            _install_from_local => \&_install_from_local,
-            _is_in_dist         => \&_is_in_dist,
-            _stderr             => \&_stderr,
-            _stdout             => \&_stdout,
-            # poe inline states
-            _start => \&_start,
-            #_stop  => sub { warn "stop " . $_[HEAP]->name . "\n"; },
-        },
-        heap => $obj,
-    );
-    return $session->ID;
+sub new {
+    my ($pkg, %params) = @_;
+    my $self = {
+        _blocking => {},
+        _missing  => {},
+        %params,
+    };
+    my $class = ref($pkg) || $pkg;
+    bless $self, $class;
+    return $self;
 }
 
 
@@ -104,12 +74,13 @@ sub spawn {
 # -- public events
 
 sub available_on_bs {
-    my ($k, $self) = @_[KERNEL, HEAP];
+    my ($k, $self) = @_[KERNEL, OBJECT];
+    $self->is_avail_on_bs(1);
     $k->post('app', 'available_on_bs', $self);
 }
 
 sub build_upstream {
-    my ($k, $self) = @_[KERNEL, HEAP];
+    my ($k, $self) = @_[KERNEL, OBJECT];
 
     # preparing command.
     my $name    = $self->name;
@@ -134,7 +105,7 @@ sub build_upstream {
 }
 
 sub cpan2dist {
-    my ($k, $self) = @_[KERNEL, HEAP];
+    my ($k, $self) = @_[KERNEL, OBJECT];
 
     # we don't want to re-build the prereqs, even if we're not at their
     # most recent version. and cpanplus --nobuildprereqs does not work
@@ -143,7 +114,7 @@ sub cpan2dist {
     # and somehow, the ignore list with regex /(?<!$name)$/ does not work.
     # so we're stuck with ignore modules one by one - sigh.
     my $ignore = '';
-    $ignore .= "--ignore '^$_\$' " foreach @{ $self->_prereqs };
+    $ignore .= "--ignore '^$_\$' " foreach @{ $self->prereqs };
 
     # preparing command. note that we do want --force, to be able to extract
     # the rpm and srpm pathes from the output.
@@ -169,7 +140,7 @@ sub cpan2dist {
 
 
 sub import_upstream {
-    my ($k, $self) = @_[KERNEL, HEAP];
+    my ($k, $self) = @_[KERNEL, OBJECT];
 
     # preparing command.
     my $name = $self->name;
@@ -194,7 +165,7 @@ sub import_upstream {
 }
 
 sub find_prereqs {
-    my ($k, $self) = @_[KERNEL, HEAP];
+    my ($k, $self) = @_[KERNEL, OBJECT];
 
     # preparing command
     my $name = $self->name;
@@ -218,7 +189,7 @@ sub find_prereqs {
 }
 
 sub install_from_dist {
-    my ($k, $self) = @_[KERNEL, HEAP];
+    my ($k, $self) = @_[KERNEL, OBJECT];
     my $name = $self->name;
 
     # check whether there's another rpm transaction
@@ -251,7 +222,7 @@ sub install_from_dist {
 }
 
 sub install_from_local {
-    my ($k, $self) = @_[KERNEL, HEAP];
+    my ($k, $self) = @_[KERNEL, OBJECT];
     my $name = $self->name;
 
     # check whether there's another rpm transaction
@@ -284,7 +255,7 @@ sub install_from_local {
 }
 
 sub is_in_dist {
-    my ($k, $self) = @_[KERNEL, HEAP];
+    my ($k, $self) = @_[KERNEL, OBJECT];
 
     # preparing command
     my $name = $self->name;
@@ -311,7 +282,7 @@ sub is_in_dist {
 
 
 sub is_installed {
-    my ($k, $self) = @_[KERNEL, HEAP];
+    my ($k, $self) = @_[KERNEL, OBJECT];
 
     my $name = $self->name;
     my $cmd  = qq{ require $name };
@@ -333,7 +304,7 @@ sub is_installed {
 # -- private events
 
 sub _build_upstream {
-    my($k, $self, $pid, $rv) = @_[KERNEL, HEAP, ARG1, ARG2];
+    my($k, $self, $pid, $rv) = @_[KERNEL, OBJECT, ARG1, ARG2];
 
     # since it's a sigchld handler, it also gets called for other
     # spawned processes. therefore, screen out processes that are
@@ -355,7 +326,7 @@ sub _build_upstream {
 
 
 sub _cpan2dist {
-    my ($k, $self, $id) = @_[KERNEL, HEAP, ARG0];
+    my ($k, $self, $id) = @_[KERNEL, OBJECT, ARG0];
     my $name = $self->name;
 
     # terminate wheel
@@ -397,7 +368,7 @@ sub _cpan2dist {
 }
 
 sub _import_upstream {
-    my($k, $self, $pid, $rv) = @_[KERNEL, HEAP, ARG1, ARG2];
+    my($k, $self, $pid, $rv) = @_[KERNEL, OBJECT, ARG1, ARG2];
 
     # since it's a sigchld handler, it also gets called for other
     # spawned processes. therefore, screen out processes that are
@@ -418,7 +389,7 @@ sub _import_upstream {
 
 
 sub _find_prereqs {
-    my ($k, $self, $id) = @_[KERNEL, HEAP, ARG0];
+    my ($k, $self, $id) = @_[KERNEL, OBJECT, ARG0];
 
     # terminate wheel
     my $wheel  = $self->_wheel;
@@ -432,7 +403,7 @@ sub _find_prereqs {
     my @prereqs = map  { (split /\s+/, $_)[0] } @wanted;
 
     # store prereqs
-    $self->_prereqs( \@prereqs );
+    $self->prereqs( \@prereqs );
     my @logs = @prereqs
         ? map { "prereq found: $_" } @prereqs
         : 'No prereqs found.';
@@ -441,7 +412,7 @@ sub _find_prereqs {
 }
 
 sub _install_from_dist {
-    my($k, $self, $pid, $rv) = @_[KERNEL, HEAP, ARG1, ARG2];
+    my($k, $self, $pid, $rv) = @_[KERNEL, OBJECT, ARG1, ARG2];
 
     # since it's a sigchld handler, it also gets called for other
     # spawned processes. therefore, screen out processes that are
@@ -465,7 +436,7 @@ sub _install_from_dist {
 
 
 sub _install_from_local {
-    my($k, $self, $pid, $rv) = @_[KERNEL, HEAP, ARG1, ARG2];
+    my($k, $self, $pid, $rv) = @_[KERNEL, OBJECT, ARG1, ARG2];
 
     # since it's a sigchld handler, it also gets called for other
     # spawned processes. therefore, screen out processes that are
@@ -490,7 +461,7 @@ sub _install_from_local {
 
 
 sub _is_in_dist {
-    my($k, $self, $pid, $rv) = @_[KERNEL, HEAP, ARG1, ARG2];
+    my($k, $self, $pid, $rv) = @_[KERNEL, OBJECT, ARG1, ARG2];
 
     # since it's a sigchld handler, it also gets called for other
     # spawned processes. therefore, screen out processes that are
@@ -507,18 +478,19 @@ sub _is_in_dist {
     my $name  = $self->name;
     my $exval = $rv >> 8;
 
+    $self->is_avail_on_bs( !$exval );
     my $status = $exval ? 'not' : 'already';
     $self->_log_result( "$name is $status packaged upstream." );
     $k->post('app', 'upstream_status', $self, !$exval);
 }
 
 sub _stderr {
-    my ($k, $self, $line) = @_[KERNEL, HEAP, ARG0];
+    my ($k, $self, $line) = @_[KERNEL, OBJECT, ARG0];
     $k->post('ui', 'append', $self, "stderr: $line\n");
 }
 
 sub _stdout {
-    my ($k, $self, $line) = @_[KERNEL, HEAP, ARG0];
+    my ($k, $self, $line) = @_[KERNEL, OBJECT, ARG0];
     $line .= "\n";
     $self->_output( $self->_output . $line );
     $k->post('ui', 'append', $self, "stdout: $line");
@@ -528,7 +500,7 @@ sub _stdout {
 # -- poe inline states
 
 sub _start {
-    my ($k, $self) = @_[KERNEL, HEAP];
+    my ($k, $self) = @_[KERNEL, OBJECT];
 
     $k->alias_set($self);
     $k->alias_set($self->name);
@@ -540,6 +512,43 @@ sub _start {
 #--
 # METHODS
 
+# -- public methods
+
+sub blocking_add {
+    my ($self, $name) = @_;
+    $self->_blocking->{$name} = 1;
+}
+
+sub blocking_clear {
+    my ($self, $name) = @_;
+    $self->_blocking({});
+}
+
+sub blocking_list {
+    my ($self) = @_;
+    my $blocking = $self->_blocking;
+    return sort keys %$blocking;
+}
+
+#
+
+sub missing_add {
+    my ($self, $name) = @_;
+    warn "$self / $name";
+    $self->_missing->{$name} = 1;
+}
+
+sub missing_del {
+    my ($self, $name) = @_;
+    delete $self->_missing->{$name};
+}
+
+sub missing_list {
+    my ($self) = @_;
+    my $missing = $self->_missing;
+    return sort keys %$missing;
+}
+
 # -- private methods
 
 sub _log_empty_line {
@@ -591,74 +600,135 @@ the module availability in the distribution.
 
 
 
-=head1 PUBLIC PACKAGE METHODS
+=head1 METHODS
 
-=head2 my $id = App::CPAN2Pkg::Module->spawn( $module )
-
-This method will create a POE session responsible for packaging &
-installing the wanted C<$module>.
-
-It will return the POE id of the session newly created.
+This package is also a class, used B<internally> to store private data
+needed for the packaging stuff.
 
 
 
-=head1 PUBLIC EVENTS ACCEPTED
+=head2 Constructor
 
-=head2 available_on_bs()
+=over 4
+
+=item my $module = App::CPAN2Pkg::Module->new(name=>$name)
+
+=back
+
+
+
+=head2 Accessors
+
+The following accessors are available:
+
+=over 4
+
+=item is_avail_on_bs() - whether the module is available on build system
+
+=item is_local() - whether the module is installed locally
+
+=item name() - the module name
+
+=item prereqs() - the module prereqs
+
+=back
+
+
+
+=head2 Public methods
+
+=over 4
+
+=item blocking_add( $module )
+
+Add C<$module> to the list of modules that current object is blocking
+from locally before trying to build
+the object.
+
+
+=item blocking_clear( $module )
+
+Remove C<$module> from the list of modules missing locally. This means that
+module has been built and installed by cpan2pkg.
+
+
+=item blocking_list( )
+
+Get the list of modules missing before trying to build the object.
+
+
+=item missing_add( $module )
+
+Add C<$module> to the list of modules missing locally before trying to build
+the object.
+
+
+=item missing_del( $module )
+
+Remove C<$module> from the list of modules missing locally. This means that
+module has been built and installed by cpan2pkg.
+
+
+=item missing_list( )
+
+Get the list of modules missing before trying to build the object.
+
+
+=back
+
+
+
+
+=head2 Public events accepted
+
+
+=over 4
+
+=item available_on_bs()
 
 Sent when module is available on upstream build system.
 
 
-=head2 build_upstream()
+=item build_upstream()
 
 Submit package to be build on upstream build system.
 
 
-=head2 cpan2dist()
+=item cpan2dist()
 
 Build a native package for this module, using C<cpan2dist> with the C<--force> flag.
 
 
-=head2 find_prereqs()
+=item find_prereqs()
 
 Start looking for any other module needed by current module.
 
 
-=head2 import_upstream()
+=item import_upstream()
 
 Try to import module into upstream distribution.
 
 
-=head2 install_from_dist()
+=item install_from_dist()
 
 Try to install module from upstream distribution.
 
 
-=head2 install_from_local()
+=item install_from_local()
 
 Try to install module from package freshly build.
 
 
-=head2 is_in_dist()
+=item is_in_dist()
 
 Check whether the package is provided by an existing upstream package.
 
 
-=head2 is_installed()
+=item is_installed()
 
 Check whether the package is installed locally.
 
 
-=head1 METHODS
-
-This package is also a class, used B<internally> to store private data
-needed for the packaging stuff. The following accessors are therefore
-available, but should not be used directly:
-
-=over 4
-
-=item name() - the module name
-
 =back
 
 
